// File:	E391a.cc
// Date:	30-Dec-2000 G.M. Dubna
// Updated:	15-Feb-2001 G.M. Juelich
// Updated:	30-Mar-2001 G.M. Juelich
// Updated:	04-Nov-2001 G.M. Dubna

#include "G4RunManager.hh"

// Close/Open Visualisation Here ...
#undef G4VIS_USE

#ifdef G4VIS_USE
#include "VisManager.hh"
#endif

#include "E391aEventWriter.hh"

#include "E391aDetectorConstruction.hh"
#include "E391aPhysicsList.hh"
#include "E391aPrimaryGeneratorAction.hh"
#include "E391aRunAction.hh"
#include "E391aEventAction.hh"
#include "E391aStackingAction.hh"
#include "E391aTrackingAction.hh"
#include "E391aSteppingAction.hh"
#include "E391aDetectorTable.hh"
#include "E391aDigitizerManager.hh"
#include "SteppingVerbose.hh"

//#include "G4UnitsTable.hh"
//#include "SystemOfUnits.hh"

#include "TROOT.h"
#include "TH1.h"
#include "TH2.h"
#include "TFile.h"
#include "TApplication.h"
#include "RunMessage.h"
#include "HistAnalysis.hh"

TROOT		Root("E391a","E391a G4 Simulation ") ;
TROOT		*gROOT=&Root ;
TFile		*HFile ;
RunMessage	*RootMsg=0 ;
HistAnalysis	*HA=0 ;

E391aEventWriter	*DataFile=0 ;
E391aDetectorTable	DT ;
E391aDigitizerManager	DM ;

#include <signal.h>

void	Interrupt(int signum) {
	(G4RunManager::GetRunManager())->AbortRun() ;
	}

int main(int argc,char** argv) {
// Ctrl-C Interrupt Handler Initialization
	signal(SIGTERM,&Interrupt) ;
	signal(SIGINT ,&Interrupt) ;
	signal(SIGPIPE,&Interrupt) ;
// Root Part
//***************************
	TApplication	App("E391aG4",&argc,argv) ;
	HFile=(TFile *)new TFile("E391aHist.root","RECREATE") ;
	RootMsg=(RunMessage *)new RunMessage ;
//***************************/
	DataFile=new E391aEventWriter() ;
//++	DataFile->Open("DataFile.bin") ; // if not needed comment ...
//***************************/
// Stepping Verbose Activation
	G4VSteppingVerbose::SetInstance(new SteppingVerbose) ;
// Geant4 Part
	G4RunManager*	RunManager=new G4RunManager ;
//***************************
	RootMsg->Time("Start Time:") ;
	RootMsg->Message("Detector Initialization ...") ;
//***************************/
	E391aDetectorConstruction* Det=new E391aDetectorConstruction ;
	RunManager->SetUserInitialization(Det) ;
	RootMsg->Message("Physics List Initialization ...") ;
	RunManager->SetUserInitialization(new E391aPhysicsList) ;

//	G4ParticleTable::GetParticleTable()->DumpTable() ;

	E391aPrimaryGeneratorAction* PGun=new E391aPrimaryGeneratorAction ;
	RunManager->SetUserAction(PGun);
	RunManager->SetUserAction(new E391aRunAction(Det,PGun)) ;
	RunManager->SetUserAction(new E391aEventAction(Det,PGun,HFile)) ;
	RunManager->SetUserAction(new E391aStackingAction) ;
	RunManager->SetUserAction(new E391aTrackingAction) ;
	RunManager->SetUserAction(new E391aSteppingAction(Det)) ;
	RunManager->SetVerboseLevel(0);
	RootMsg->Message("User Action Initialization ...") ;
	RunManager->Initialize();
#ifdef G4VIS_USE
//+	VisManager* visManager=new VisManager(Det->World()) ;
#endif
// Activate Run ...
	RootMsg->Message("BeamOn ...") ;
//	RunManager->SetVerboseLevel(6) ; // Only for run manager
//	RunManager->BeamOn(1000) ;
	RunManager->BeamOn(1000000) ;
//+	visManager->DetDraw() ;
// Finish ...
	delete RunManager ;
#ifdef G4VIS_USE
//+	delete visManager ;
#endif
//***************************
	HA=new HistAnalysis ;
	HA->ER(gROOT) ;
	HFile->Write(0,TObject::kOverwrite) ;
	HFile->Close() ;
	delete HA ;
	delete RootMsg ;
	delete HFile ;
	delete DataFile ;
//***************************/

	return 0 ;

	}

// EOF
