// File:	E391aCalorimeterFilter.hh
// Date:	10-May-2001 G.Macharashvili Dubna
// Date:	20-Jun-2001 G.Macharashvili Dubna

#ifndef E391aCalorimeterFilter_h
#define E391aCalorimeterFilter_h 1

#include "E391aEventWriter.hh"

#include "E391aPrimaryGeneratorAction.hh"
#include "E391aCalorimeterSD.hh"
#include "E391aHit.hh"
#include "E391aDigi.hh"
#include "DynamicGamma.hh"
#include "SortedMap.hh"

#include "G4VDigitizerModule.hh"
#include "G4Event.hh"
#include "G4HCofThisEvent.hh"
#include "G4DCofThisEvent.hh"
#include "G4SDManager.hh"
#include "G4DigiManager.hh"

#include "CLHEP/Random/RandGaussQ.h"

#include "TH1.h"
#include "TH2.h"

class E391aDetectorConstruction ;

class E391aCalorimeterFilter {

static	const	G4double	Zc=6020*mm ; // ECalorimeter face coord
static	const	G4double	QDCScale=4*MeV ; // Calibration Constant
static	const	G4double	CCSigma=0.02 ; // In percent
static	const	G4double	EgCut=0*MeV ; // Minimal Cut For Gamma Er
static	const	G4bool		CorrON=true ; // Coordinate Correction ON(true)/OFF(false)

public:

	E391aCalorimeterFilter(E391aDetectorConstruction *,E391aCalorimeterConfig *) ;
	~E391aCalorimeterFilter() ;

	void		Init(const G4Event*) ;
	void		ProcessEvent(E391aPrimaryGeneratorAction *Gen) ;
	void		End() ;
	G4int		FindMax() ;
inline	G4double	Cnt(G4double x) ;
inline	G4double	Loc(G4double x) ;
	G4double	Corr(G4double x) ;
	Gamma*		GammaAroundMax(G4int kmax) ;
	Gamma*		GammaAroundMax2(G4int kmax) ;
	void		GammaSearch() ;
	G4double	KLongDecayPointZ(Gamma *,Gamma *) ;
	G4double	AcoplanarityAngle(Gamma *,Gamma *) ;
	G4double	PionPT(Gamma *g1,Gamma *g2) ;
	G4double	Distance(Gamma *g1,Gamma *g2) ;

private:

	E391aDetectorConstruction	*Det ;
	E391aCalorimeterConfig		*Config ;
	G4SDManager			*SDMan ;
	G4DigiManager			*DM ;

	G4HCofThisEvent			*HCE ;
	G4int				HiCID ;
	G4int				SBarrelHCID ;
	G4int				BBarrelHCID ;
	E391aHitsCollection		*HC ;
	E391aHitsCollection		*SBHC ;
	E391aHitsCollection		*BBHC ;

	G4int				NumOfHitsEC ;
	G4int				NumOfHitsSB ;
	G4int				NumOfHitsBB ;
	G4int				NumOfDigits ;

static	const	G4int			NMaxD=200 ;
	G4int				UsedD[NMaxD] ;
	G4int				NumOfUsedD ;

	G4DCofThisEvent			*DCE ;
	E391aDigiCollection		*DC ;
	G4int				DiCID ;
	E391aDigiCollection		*CHDC ;
	G4int				CHDiCID ;

	G4double			*cc ;
	SortedMap<Gamma>		Gin ;
	SortedMap<Gamma>		G ;
	G4double			Dist , r1 , r2 ;

	TH1F				*ECSp,*HEo,*HEr,*HErS,*HXrS,*HYrS ;
	TH2F				*HEoEr,*HEoRes,*HXoXr,*HYoYr ;

	TH2F				*HPrSAc,*HAcSPr,*HPrSAcS ;
	TH2F				*HPTRes,*HAcoRes ;
	TH1F				*HPTS,*HAcoS,*HNDig ;
	TH1F				*HPErS,*HPgg,*HIgg ;
	TH2F				*HPptAco,*HIptAco,*HZoZr ;

	TH2F				*HXY,*HXYr,*HDxDy,*HDxES,*HDyES ;
	TH1F				*HDx,*HDy,*HDxo,*HDyo ;
	TH1F				*HZr,*HZrS ;

	TH1F				*ZHitXP,*ZHitYP ;
	TH2F				*ZHitXY ;
	TH1F				*HQQ,*HQT,*HQQD,*HQTD,*HQHD ;

	} ;

G4double E391aCalorimeterFilter::Cnt(G4double GlobX) {
	return Config->XCenter(GlobX) ;
	}
	
G4double E391aCalorimeterFilter::Loc(G4double GlobX) {
	return GlobX-Cnt(GlobX) ;
	}
	
#endif

// EOF
