// File:	E391aHit.hh
// Date:	30-Aug-2001 G.Macharashvili Dubna
// Date:	24-Sep-2001 G.Macharashvili Juelich

#ifndef E391aHit_h
#define E391aHit_h 1

#include "E391aPType.hh"

#include "G4VHit.hh"
#include "G4THitsCollection.hh"
#include "G4ThreeVector.hh"
#include "G4Step.hh"

class E391aHit : public G4VHit {

public:

		~E391aHit() ;
		E391aHit(G4int DetectorID=0,G4int ParticleID=-1,G4double Eo=0,G4double E=0,G4double T=0) ;
		E391aHit(const E391aHit &r) ;
		E391aHit(G4int id,G4int pid,G4Step *s) ;
const	E391aHit&	operator=(const E391aHit &r) ;

	int	operator==(const E391aHit &r) const ;
inline	void	*operator new(size_t) ;
inline	void	operator delete(void *h) ;
	void	Draw() {}
	void	Print() ;

public:

inline	void		SetDetectorID(G4int id) { xDetectorID=id ; }
inline	G4int		DetectorID() { return xDetectorID ; }
inline	G4int		SetParticleID(G4int id) { xParticleID=id ; }
inline	G4int		ParticleID() { return xParticleID ; }

inline	void		SetEDep(G4double de) { xEDep=de ; }
inline	void		AddEDep(G4double de) { xEDep+=de ; }
inline	G4double	EDep() { return xEDep ; }

inline	void		SetKineticEnergy(G4double e) { xEKin=e ; }
inline	void		AddKineticEnergy(G4double e) { xEKin+=e ; }
inline	G4double	KineticEnergy() { return xEKin ; }

inline	void		SetTime(G4double t) { xTime=t ; }
inline	G4double	Time() { return xTime ; }

inline	void		SetPos(G4ThreeVector x) { xPos=x ; }
inline	void		SetDir(G4ThreeVector x) { xDir=x ; }
inline	G4double	X() { return xPos.x() ; }
inline	G4double	Y() { return xPos.y() ; }
inline	G4double	Z() { return xPos.z() ; }
inline	G4double	XPrime() { return (xDir.z()>0) ? xDir.x()/xDir.z() : 0 ; }
inline	G4double	YPrime() { return (xDir.z()>0) ? xDir.y()/xDir.z() : 0 ; }

inline	void		SetPath(G4double p) { xPath=p ; }
inline	void		AddPath(G4double p) { xPath+=p ; }
inline	G4double	Path() { return xPath ; }

inline	void		Add(G4Step *s) ;

private:

	G4int		xDetectorID ;
	G4int		xParticleID ;	
	G4double	xTime ;		// Global Time. (depends on xEDep!!)
	G4double	xEKin ;		// Kinetic energy at 'xPos'
	G4ThreeVector	xPos ;		// Particle Position (Glob.sys)
	G4ThreeVector	xDir ;		// Particle Momentum Direction (Glob.sys)
	G4double	xEDep ;		// Deposited in the Medium Energy
	G4double	xPath ;		// Summary Track Length so far

	} ;

typedef	G4THitsCollection<E391aHit>	E391aHitsCollection ;

extern	G4Allocator<E391aHit>	E391aHitsAllocator ;

inline	void*	E391aHit::operator new(size_t) {
	return (void *)E391aHitsAllocator.MallocSingle() ;
	}

inline	void	E391aHit::operator delete(void *H) {
	E391aHitsAllocator.FreeSingle((E391aHit*) H);
	}

#endif

// EOF
