// File:	E391aCCo4.cc
// Date:	24-Sep-2001 G.Macharashvili Juelich
// Updated:	22-Oct-2001 G.Macharashvili Juelich

#include "E391aCCo4.hh"

G4String	E391aCCo4::Name="CCo4" ;

	E391aCCo4::E391aCCo4():Sol(0),Log(0),Phy(0),World(0),CsI(0),CeF3(0),Dig(0) {
#ifdef HIST_ON
	DH=0 ;
#endif
	}
	
	E391aCCo4::~E391aCCo4() {
#ifdef HIST_ON
	if (DH) delete DH ;
#endif
	}
	
G4LogicalVolume*	E391aCCo4::Construct(G4VPhysicalVolume *W) {
	World=W ;
	Materials() ;
	Geometry() ;
	InfraStructure() ;
	Detector() ;
	Digitizer() ;
	Visualization() ;
#ifdef HIST_ON
	DH=new E391aDetectorHistograming(Name) ;
#endif
	return Log ;
	}

void	E391aCCo4::Geometry() {
	Sol=new G4Tubs(Name,BeamHoleRadius,OuterRadius,(BackPlaneZ-FrontEndZ)/2,0,360*deg) ;
	Log=new G4LogicalVolume(Sol,CsI,Name,0,0,0) ;
	Phy=new G4PVPlacement(0,G4ThreeVector(0,0,(FrontEndZ+BackPlaneZ)/2),Name,Log,World,false,0) ;

//	Log->SetUserLimits(new G4UserLimits(DBL_MAX,DBL_MAX,1000*ns,1.*MeV,1*mm)) ;
	}

void	E391aCCo4::InfraStructure() {}

void	E391aCCo4::Detector() {
// Default Detector ... (Kills all track immdiately)
//	Det=new DummyDetector(Name) ; // Alternative detector. for Fast run ...
	Det=new E391aCCo4SensitiveDetector(Name) ;
	G4SDManager::GetSDMpointer()->AddNewDetector(Det) ;
	Log->SetSensitiveDetector(Det) ;
	}

void	E391aCCo4::Digitizer() {
//	Dig=new E391aCCo4Digitizer(Name) ;
//	G4DigiManager::GetDMpointer()->AddNewModule(Dig) ;
	}

void	E391aCCo4::Visualization() {
static	const	G4Color	CCo4Color(0.4,0.6,1.0) ;
	G4VisAttributes	*VisAtt=new G4VisAttributes(Visible,CCo4Color) ;
	VisAtt->SetForceSolid(true) ;
	Log->SetVisAttributes(VisAtt) ;
	}

void	E391aCCo4::Materials() {
	if (CsI==0) {
		G4Element *Cs=new G4Element("Cesium", "Cs", 55, 132.9054*g/mole) ;
		G4Element *I =new G4Element("Iodine", "I",  53, 126.904 *g/mole) ;
		CsI=new G4Material("CsI",4.51*g/cm3,2) ;
			CsI->AddElement(I, .5) ;
			CsI->AddElement(Cs,.5) ;
		}
 
	CeF3=G4Material::GetMaterial("CeF3") ;
	if (CeF3==0) {
		G4Element *Ce=new G4Element("Cerium", "Ce", 58, 140.116 *g/mole) ;
		G4Element *F= new G4Element("Fluor",  "F",   9,  18.998 *g/mole) ;
		CeF3=new G4Material("CeF3",6.16*g/cm3,2) ;
			CeF3->AddElement(F, 3) ;
			CeF3->AddElement(Ce,1) ;
		}
 	}

// EOF
