// File:	E391aCH-Digi.cc
// Date:	20-Sep-2001 G.Macharashvili Juelich
// Updated:

#include "E391aCH-Digi.hh"

E391aCHDigitizer::~E391aCHDigitizer() { delete [] DigiID ; }

E391aCHDigitizer::E391aCHDigitizer(G4String Name) : E391aVDigitizerModule(Name) {
	DigiID=new G4int[xMaxDigi] ;
//	collectionName.insert(Name) ; // G4 before v.3
	collectionName.insert(collectionName.begin(),Name) ; // G4 v.3 and later ...
	}

G4double E391aCHDigitizer::ERand(G4double e) {
	return (ESigmaON) ? RandGaussQ::shoot(e,0.01*e) : e ;
	}

G4double E391aCHDigitizer::PedestalRand() { 
	return (PedSigmaON) ? rint(RandGaussQ::shoot(0,QDCPedSigma)) : 0 ;
	}

G4double E391aCHDigitizer::QDC(G4double e) {
	double	Q=(ESigmaON) ? rint(e*MeV2Npe+PedestalRand()) : e/MeV2Npe ;
	return (Q>0) ? Q : 0 ;
	}

void	E391aCHDigitizer::DigiFill(G4int id,G4double Q,G4double T) {
	if (Q<0.99) return ;
	G4int k=id%10000 ;
	if (DigiID[k]==-1) DigiID[k]=DC->insert(new E391aDigi(id,Q,T))-1 ;
	else (*DC)[DigiID[k]]->AddQ(Q) ;
	}

void	E391aCHDigitizer::MakeDigi(E391aHit *H) {
// corrections needed ...
	G4int id=H->DetectorID() ;
// gamma ...
	if (H->ParticleID()==0) { // Gamma ...
		G4double E=ERand(H->EDep()) ;
		DigiFill(id,QDC(E),TDC(H->Time())) ;
		}
// charged. deposited energy ...
	else if (H->ParticleID()>=1 && H->ParticleID()<7) {
		G4double E=ERand(H->EDep()) ;
		DigiFill(id,QDC(E),TDC(H->Time())) ;
		}
// neutrinos ...
	else if (H->ParticleID()>7) {}
// error for pio ...
	else {}
	}
			
void E391aCHDigitizer::Digitize() {
	int k ;
static	G4int	HiCID=-1 ;
	if (HiCID==-1) HiCID=G4DigiManager::GetDMpointer()->GetHitsCollectionID(moduleName) ;
	if (HiCID<0) return ;
	HC=(E391aHitsCollection*)G4DigiManager::GetDMpointer()->GetHitsCollection(HiCID) ;
	if (HC==0) return ; // No Hits For CH ...
	G4int	NumOfHits=HC->entries() ;
	if (NumOfHits==0) return ; // 
// Hit control print ...
//+	for (k=0 ; k<NumOfHits ; k++) (*HC)[k]->Print() ;
	Digitize(HC) ;
//+	G4int	NumOfDigits=DC->entries() ;
//+	G4cout.form("NumOfDigits: %d\n",NumOfDigits) ;
//+	for (k=0 ; k<NumOfDigits ; k++) (*DC)[k]->Print() ;

	}

void E391aCHDigitizer::Digitize(E391aHitsCollection *HC) {
//	G4cout<<moduleName<<" ..." << endl ;
	G4int k , NumOfHits=HC->entries() ;
	for(k=0 ; k<xMaxDigi ; k++) DigiID[k]=-1 ;
	DC=new E391aDigiCollection(moduleName.data(),collectionName[0]) ;
	StoreDigiCollection(DC) ;
	for (k=0 ; k<NumOfHits ; k++) MakeDigi((*HC)[k]) ;
	}

// EOF
