// File:	E391aCH-SD.cc
// Date:	06-Sep-2001 G.Macharashvili Dubna
// Updated:	

#include "E391aCH-SD.hh"

E391aChargedVetoDetector::E391aChargedVetoDetector(
	G4String DetName,
	E391aDetectorConstruction* det) :
		G4VSensitiveDetector(DetName),xDet(det) {
		collectionName.insert(DetName) ;
		HitID=new G4int[xHitMax] ;

	CHSpectG=new TH1F("CHSpectG","#gamma Energy in ChargedVeto",200,0,0.4) ;
		CHSpectG->SetXTitle("#gamma energy,  GeV") ;
		CHSpectG->SetYTitle("Number of events") ;
	CHSpectE=new TH1F("CHSpectE","e^{#pm} Energy in ChargedVeto",200,0,0.4) ;
		CHSpectE->SetXTitle("energy,  GeV") ;
		CHSpectE->SetYTitle("Number of events") ;
	CHEDep=new TH1F("CHEDep","Deposited Energy in ChargedVeto",100,0,50) ;
		CHEDep->SetXTitle("Deposited energy,  MeV") ;
		CHEDep->SetYTitle("Number of events") ;
	CHHitXY=new TH2F("CHHitXY","Hit point distribution in ChargedVeto",100,-950,+950,100,-950,+950) ;
		CHHitXY->SetXTitle("X coordinate,  mm") ;
		CHHitXY->SetXTitle("Y coordinate,  mm") ;

		}

E391aChargedVetoDetector::~E391aChargedVetoDetector() { delete [] HitID ; }

void E391aChargedVetoDetector::Initialize(G4HCofThisEvent* HCE) {
	HC=new E391aHitsCollection(SensitiveDetectorName,collectionName[0]) ;
	for(G4int j=0 ; j<xHitMax ; j++) HitID[j]=-1 ;
	}

G4bool E391aChargedVetoDetector::ProcessHits(G4Step* S,G4TouchableHistory* RO) {
	G4StepPoint	*SP=S->GetPreStepPoint() ;
// Scintillator bar copyNo ...
	module=SP->GetPhysicalVolume()->GetCopyNo() ;
	ParID=E391aPType::PType(S->GetTrack()->GetDefinition()) ;
	TrackID=S->GetTrack()->GetTrackID() ;
	if (module>=xHitMax) return false ;
	if (HitID[module]==-1) {
		E391aHit* Hit=new E391aHit(CHDetectorID*10000+module,ParID,S) ;
		HitID[module]=HC->insert(Hit)-1 ;
// Hist Fill for 'primary' Hit ...
		if (ParID==0) {
			CHSpectG->Fill(SP->GetKineticEnergy()/GeV) ;
			CHHitXY->Fill(SP->GetPosition().x(),SP->GetPosition().y()) ;
			}
		if (ParID==1||ParID==2)
			CHSpectE->Fill(SP->GetKineticEnergy()/GeV) ;
		}
// 'secondary' Hit in the same module ...
	else (*HC)[HitID[module]]->AddEDep(S->GetTotalEnergyDeposit()) ;

	return true ;

	}

void	E391aChargedVetoDetector::EndOfEvent(G4HCofThisEvent* HCE) {
// Hist Fill ...
	for(G4int j=0 ; j<xHitMax ; j++) {
		if (HitID[j]!=-1) {
			if ((*HC)[HitID[j]]->ParticleID()!=0) {
				CHEDep->Fill((*HC)[HitID[j]]->EDep()) ;
				}
			}
		}
// ...
static	G4int	HCID=-1 ; // Collection idetifier ...
	if (HCID<0) HCID=G4SDManager::GetSDMpointer()->GetCollectionID(collectionName[0]) ;
	HCE->AddHitsCollection(HCID,HC) ;
	}

// EOF
