// File:	E391aCalorimeterDigitizer.cc
// Date:	14-May-2001 G.Macharashvili Dubna
// Updated:	12-Jun-2001 G.Macharashvili Dubna
// Updated:	26-Jun-2001 G.Macharashvili Dubna

#include "E391aCalorimeterDigitizer.hh"

E391aCalorimeterDigitizer::~E391aCalorimeterDigitizer() { delete [] DigiID ; }

E391aCalorimeterDigitizer::E391aCalorimeterDigitizer(G4String Name,E391aCalorimeterConfig* cfg)
	:
	E391aVDigitizerModule(Name),Config(cfg) {
	DigiID=new G4int[Config->NumOfX()*Config->NumOfY()] ;
	collectionName.insert(collectionName.begin(),Name) ;
	}

void	E391aCalorimeterDigitizer::PartialE(G4double x,G4double xp,G4double xc,G4double e[]) {
const	G4double	b=15*mm ;
// b (the exponent slope) is calculated from Moliere Radius of CsI (38 mm).
// b=-Rm/ln(0.1)=16.5 mm !
// The same should be used at reconstructed coordinate correction.
	e[0]=0.5*exp(-(0.5*Config->DD()+(x-xc))/b)*(1-xp) ;
	e[2]=0.5*exp(-(0.5*Config->DD()-(x-xc))/b)*(1+xp) ;
	e[1]=1-e[0]-e[2] ;
	}

G4double E391aCalorimeterDigitizer::ERand(G4double e) {
	return (ESigmaON) ? RandGaussQ::shoot(e/GeV,sqrt(e/GeV)*Ea+Eb*(e/GeV))*GeV : e ;
	}

G4double E391aCalorimeterDigitizer::PedestalRand() { 
	return (PedSigmaON) ? rint(RandGaussQ::shoot(0,QDCPedSigma)) : 0 ;
	}

G4double E391aCalorimeterDigitizer::QDC(G4double e) {
	double	Q=(ESigmaON) ? rint(e/QDCScale+PedestalRand()) : e/QDCScale ;
	return (Q>0) ? Q : 0 ;
	}

void	E391aCalorimeterDigitizer::DigiFill(G4int ix,G4int iy,G4double Q,G4double T) {
	if (Q<0.99) return ;
	if (!Config->Test(ix,iy)) return ;
	int k=Config->Index(ix,iy) ;
	if (DigiID[k]==-1) {
		E391aDigi* D=new E391aDigi(Config->Ident(ix,iy),Q,T) ;
		DigiID[k]=DC->insert(D)-1 ;
		}
	else (*DC)[DigiID[k]]->AddQ(Q) ;
	}

void
E391aCalorimeterDigitizer::EMShowerModel(E391aHit *H) {
	G4int	i , j ;
	G4int	ix=Config->XNo(H->DetectorID()) ;
	G4int	iy=Config->YNo(H->DetectorID()) ;
	G4double	ex[3] , ey[3] , E ;
// gamma, e+, e- EM shower model ...
	if (H->ParticleID()<3) {
		PartialE(H->X(),H->XPrime(),Config->XCenter(ix),ex) ;
		PartialE(H->Y(),H->YPrime(),Config->YCenter(iy),ey) ;
		for (i=ix-1 ; i<=ix+1 ; i++) {
			for (j=iy-1 ; j<=iy+1 ; j++) {
				E=H->EDep()*ex[i-ix+1]*ey[j-iy+1] ;
				E=ERand(E) ;
				DigiFill(i,j,QDC(E),TDC(H->Time())) ;
				}
			}
		}
// charged. deposited energy ...
	else if (H->ParticleID()>=3 && H->ParticleID()<7) {
//		H->Print() ;
		E=ERand(H->EDep()) ;
		DigiFill(ix,iy,QDC(E),TDC(H->Time())) ;
		}
// neutrinos ...
	else if (H->ParticleID()>7) {}
// error for pio ...
	else {}
	}
			
void E391aCalorimeterDigitizer::Digitize() {
	int k ;
static	G4int	HiCID=-1 ;
	if (HiCID==-1) HiCID=G4DigiManager::GetDMpointer()->GetHitsCollectionID(moduleName) ;
	HC=(E391aHitsCollection*)G4DigiManager::GetDMpointer()->GetHitsCollection(HiCID) ;
	if (HC==0) return ; // No Hits For Calorimeter ...
	G4int	NumOfHits=HC->entries() ;
	if (NumOfHits==0) return ; // 
// Hit control print ...
//+	for (k=0 ; k<NumOfHits ; k++) (*HC)[k]->Print() ;
	Digitize(HC) ;
	}

void E391aCalorimeterDigitizer::Digitize(E391aHitsCollection* HC) {
//	G4cout<<moduleName<<" ..." << endl ;
	G4int k , NumOfHits=HC->entries() ;
	for(k=0 ; k<Config->NumOfX()*Config->NumOfY() ; k++) DigiID[k]=-1 ;
	DC=new E391aDigiCollection(moduleName.data(),collectionName[0]) ;
	StoreDigiCollection(DC) ;
	for (k=0 ; k<NumOfHits ; k++) EMShowerModel((*HC)[k]) ;
	}

// EOF
