// File:	E391aCalorimeterFilter.cc
// Date:	14-May-2001 G.Macharashvili Dubna
// Date:	20-Jun-2001 G.Macharashvili Dubna

#include "E391aCalorimeterFilter.hh"

//extern	E391aEventWriter	*DataFile ;

E391aCalorimeterFilter::~E391aCalorimeterFilter() {}

E391aCalorimeterFilter::E391aCalorimeterFilter(E391aDetectorConstruction* det,E391aCalorimeterConfig *cfg) {
//	G4cout.form("E391aCalorimeterFilter::new \n") ;
	Det=det ;
	Config=cfg ;
	SBarrelHCID=BBarrelHCID=HiCID=-2 ;
	HC=0 , SBHC=0 , BBHC=0 ;
	HCE=0 ;
	DiCID=-2 ;
	CHDiCID=-2 ;
	DCE=0 ;
	SDMan=0 ;
	DM=0 ;
	CHDC=0 ;
	DC=0 ;
// Calibration constants for all modules:
	cc=new G4double[Config->NumOfX()*Config->NumOfY()] ;
	for (int i=0 ; i<Config->NumOfX() ; i++) {
		for (int j=0 ; j<Config->NumOfY() ; j++) {
			cc[Config->Index(i,j)]=RandGaussQ::shoot(QDCScale,CCSigma*QDCScale) ;
			}
		}

	G4double emax=3 ;
	ECSp	=new TH1F("ECSp","EC Hitted #gamma Energy",200,0,0.4) ;
		ECSp->SetXTitle("#gamma energy,  GeV") ;
		ECSp->SetYTitle("Number of events") ;

	HEo	=new TH1F("HEo","Gamma Energy",100,0,emax) ;
		HEo->SetXTitle("Generated gamma energy,  GeV") ;
		HEo->SetYTitle("Number of events") ;
	HEr	=new TH1F("HEr","Gamma E(R)",100,0,emax) ;
		HEr->SetXTitle("Reconstructed gamma energy,  GeV") ;
		HEr->SetYTitle("Number of events") ;
	HErS	=new TH1F("HErS","Gamma E(R)-Gamma E(G)",100,-0.6,+0.4) ;
		HErS->SetXTitle("Gamma energy resolution, GeV") ;
		HErS->SetYTitle("Number of events") ;
	HEoEr	=new TH2F("HEoEr","Gamma E(G) vs Gamma E(R)",100,0,emax,100,0,emax) ;
		HEoEr->SetXTitle("Generated gamma energy,  GeV") ;
		HEoEr->SetYTitle("Reconstructed gamma energy,  GeV") ;
	HEoRes	=new TH2F("HEoRes","Gamma: (E_{r}-E_{g})/E_{g} vs E_{g}",30,0,emax,60,-0.6,0.6) ;
		HEoRes->SetXTitle("Generated gamma energy,  GeV") ;
		HEoRes->SetYTitle("#gamma energy relative precision,  #sigma/E") ;
	HXrS	=new TH1F("HXrS","Gamma: X^{R} - X^{G}",100,-25,+25) ;
		HXrS->SetXTitle("Coordinate resolution, mm") ;
		HXrS->SetYTitle("Number of events") ;
	HYrS	=new TH1F("HYrS","Gamma: Y^{Y} - Y^{G}",100,-25,+25) ;
		HYrS->SetXTitle("Coordinate resolution, mm") ;
		HYrS->SetYTitle("Number of events") ;
	HDx	=new TH1F("HDx","Cluster Dispersion D_{x}",120,0,60) ;
		HDx->SetXTitle("Cluster dispersion, mm") ;
		HDx->SetYTitle("Number of events") ;
	HDy	=new TH1F("HDy","Cluster Dispersion D_{y}",120,0,60) ;
		HDy->SetXTitle("Cluster dispersion, mm") ;
		HDy->SetYTitle("Number of events") ;
	HDxo	=new TH1F("HDxo","Cluster Dispersion (x)+cutEdge",120,0,60) ;
	HDyo	=new TH1F("HDyo","Cluster Dispersion (y)+cutEdge",120,0,60) ;
	HDxDy	=new TH2F("HDxDy","Cluster D_{x} vs Cluster D_{y}",120,0,60,120,0,60) ;
		HDxDy->SetXTitle("Cluster X dispersion, mm") ;
		HDxDy->SetYTitle("Cluster Y dispersion, mm") ;
	HXY	=new TH2F("HXY","Cluster X^{G} vs Cluster y^{G}",100,-100,100,100,-100,100) ;
		HXY->SetXTitle("X cluster center, cm") ;
		HXY->SetYTitle("Y cluster center, cm") ;
	HXYr	=new TH2F("HXYr","Cluster X^{R} vs Cluster y^{R}",100,-100,100,100,-100,100) ;
		HXYr->SetXTitle("X cluster center, cm") ;
		HXYr->SetYTitle("Y cluster center, cm") ;
	HDxES	=new TH2F("HDxES","Cluster Disp x vs #sigma E",120,0,60,100,-0.6,0.4) ;
	HDyES	=new TH2F("HDyES","Cluster Disp y vs #sigma E",120,0,60,100,-0.6,0.4) ;
	HXoXr	=new TH2F("HXoXr","Gamma: X(G) vs X(R)",140,-35,+35,140,-35,+35) ;
		HXoXr->SetXTitle("Generated coordinate, mm") ;
		HXoXr->SetYTitle("Reconstructed coordinate, mm") ;
	HYoYr	=new TH2F("HYoYr","Gamma: Y(G) vs Y(R)",140,-35,+35,140,-35,+35) ;
		HYoYr->SetXTitle("Generated coordinate, mm") ;
		HYoYr->SetYTitle("Reconstructed coordinate, mm") ;
	HPTRes	=new TH2F("HPTRes","#pi^{o} PT resolution",125,0,250,125,0,250) ;
		HPTRes->SetXTitle("Generated #pi^o P_{T},  MeV/c") ;
		HPTRes->SetYTitle("Reconstructed #pi^o P_{T},  MeV/c") ;
	HAcoRes	=new TH2F("HAcoRes","Acop.Angle resolution",90,0,180,90,0,180) ;
		HAcoRes->SetXTitle("Generated acoplanarity angle, deg") ;
		HAcoRes->SetYTitle("Reconstructed acoplanarity angle, deg") ;
	HPTS	=new TH1F("HPTS","Pion P_{T} (Rec - Gen)",100,-40,+40) ;
		HPTS->SetXTitle("Pion P_{T} resolution,  MeV/c") ;
		HPTS->SetYTitle("Number of events") ;
	HAcoS	=new TH1F("HAcoS","Acopl.Angle (Rec - Gen)",100,-10,+10) ;
		HAcoS->SetXTitle("Acoplanarity angle resolution,  deg") ;
		HAcoS->SetYTitle("Number of events") ;
	HIgg	=new TH1F("HIgg","Distance between 2 gammas (G)",200,0,200) ;
		HIgg->SetXTitle("Generated distance,  cm") ;
		HIgg->SetYTitle("Number of events") ;
	HIptAco	=new TH2F("HIptAco","Pion PT(G) vs Acop.Angle(G)",125,0,250,90,0,180) ;
	HPrSAc	=new TH2F("HPrSAc ","P_{T} resol vs Acopl.Angle(r)",40,-40,+40,45,0,+180) ;
	HPrSAcS	=new TH2F("HPrSAcS","P_{T} resol vs Acopl.Angle resol",50,-25,+25,50,-25,+25) ;
	HAcSPr	=new TH2F("HAcSPr ","Acopl.Angle resol vs PT",50,-25,+25,50,0,250) ;
	HPgg	=new TH1F("HPgg","Distance between 2 gammas (R)",200,0,200) ;
		HPgg->SetXTitle("Reconstructed distance,  cm") ;
		HPgg->SetYTitle("Number of events") ;
	HPptAco	=new TH2F("HPptAco","Pion PT(R) vs Acop.Angle(R)",125,0,250,90,0,180) ;
	HPErS	=new TH1F("HPErS","Pion E(R)-Pion E(G)",100,-0.6,+0.4) ;
		HPErS->SetXTitle("Pion energy resolution,  GeV") ;
		HPErS->SetYTitle("Number of events") ;
	HNDig	=new TH1F("HNDig","Number of generated Digi's",40,0,40) ;
		HNDig->SetXTitle("Number of generated digi objects") ;
		HNDig->SetYTitle("Number of events") ;
	HZr	=new TH1F("HZr","K_{L}^{o} Decay Point Z_{R}",200,250,650) ;
		HZr->SetXTitle("Reconstructed decay point,  cm") ;
		HZr->SetYTitle("Number of events") ;
	HZrS	=new TH1F("HZrS","K_{L}^{o} decay Z_{R} - Z_{G}",100,-100,+100) ;
		HZrS->SetXTitle("Decay point resolution,  cm") ;
		HZrS->SetYTitle("Number of events") ;
	HZoZr	=new TH2F("HZoZr","Decay Z_{G} vs Decay Z_{R}",100,200,600,100,200,600) ;
		HZoZr->SetXTitle("Generated decay point,  cm") ;
		HZoZr->SetYTitle("Reconstructed decay point,  cm") ;

	ZHitXY	=new TH2F("ZHitXY","Hit XY ",100,-100,100,100,-100,100) ;
	ZHitXP	=new TH1F("ZHitXP","Hit X Prime ",100,0,1) ;
	ZHitYP	=new TH1F("ZHitYP","Hit Y Prime ",100,0,1) ;

	HQQ	=new TH1F("HQQ","Q Raw",100,0,800) ;
		HQQ->SetXTitle("Raw Spectrum,  ch.") ;
		HQQ->SetYTitle("Number of events") ;
	HQT	=new TH1F("HQT","T Raw",100,0,500) ;
		HQT->SetXTitle("Raw Spectrum,  ch.") ;
		HQT->SetYTitle("Number of events") ;
	HQQD	=new TH1F("HQQD","Energy difference of 2 #gamma's",100,-4,1) ;
		HQQD->SetXTitle("E(#gamma1) - E(#gamma2),  GeV") ;
		HQQD->SetYTitle("Number of events") ;
	HQTD	=new TH1F("HQTD","Time difference of 2 #gamma's",100,-50,50) ;
		HQTD->SetXTitle("TDC's difference,  ch.") ;
		HQTD->SetYTitle("Number of events") ;
	HQHD	=new TH1F("HQHD","#Delta#theta difference of 2 #gamma's",100,-50,50) ;
		HQHD->SetXTitle("#theta_{#gamma1} - #theta_{#gamma2},  deg") ;
		HQHD->SetYTitle("Number of events") ;
	}

void E391aCalorimeterFilter::Init(const G4Event *Evt) {
	int k ;
//	G4cout.form("CalorimeterFilter::Init() \n") ;
//G4cout.form("1") ; G4cout.flush() ;
	if (SDMan==0) 		SDMan=G4SDManager::GetSDMpointer() ;
	if (DM==0) 		DM=G4DigiManager::GetDMpointer() ;
	if (HCE==0)		HCE=Evt->GetHCofThisEvent() ;
	if (DCE==0)		DCE=Evt->GetDCofThisEvent() ;
// Initialization ...
	for (k=0 ; k<NMaxD ; k++) UsedD[k]=0 ;
	NumOfUsedD=0 ;
	Dist=0 ;
// ...  //////////////////////////////////////
//	DataFile->Init(44,4) ;
// ...  //////////////////////////////////////
// Init End ...

	if (SBarrelHCID==-2) SBarrelHCID=SDMan->GetCollectionID("SB") ;
	if (BBarrelHCID==-2) BBarrelHCID=SDMan->GetCollectionID("BB") ;
	if (HiCID==-1) return ;
	if (HiCID==-2) HiCID=SDMan->GetCollectionID("EC") ;
	if (HiCID<0) return ;
	HC=(E391aHitsCollection*)(HCE->GetHC(HiCID)) ;
	if (HC==0) return ;
	SBHC=(E391aHitsCollection*)(HCE->GetHC(SBarrelHCID)) ;
	BBHC=(E391aHitsCollection*)(HCE->GetHC(BBarrelHCID)) ;

	NumOfHitsEC=HC->entries() ;
	if (NumOfHitsEC==0) return ; // No Hits at Calorimeter ...

//	G4cout.form("NumOfHitsEC: %d\n",NumOfHitsEC) ;
//	for (k=0 ; k<NumOfHitsEC ; k++) (*HC)[k]->Print() ;

	if (DiCID==-2) DiCID=DM->GetDigiCollectionID("EC") ;
	if (DiCID>=0) DC=(E391aDigiCollection*)DCE->GetDC(DiCID) ;
	if (CHDiCID==-2) CHDiCID=DM->GetDigiCollectionID("CH") ;
	if (CHDiCID>=0) CHDC=(E391aDigiCollection*)DCE->GetDC(CHDiCID) ;
	NumOfDigits=(DC) ? DC->entries() : 0 ;
// ...  //////////////////////////////////////
//	if (DC)   for (k=0 ; k<DC->entries() ; k++) DataFile->AddData((*DC)[k]) ;
//	if (CHDC) for (k=0 ; k<CHDC->entries() ; k++) DataFile->AddData((*CHDC)[k]) ;
// ...  //////////////////////////////////////		
	HNDig->Fill(double(NumOfDigits)) ;
	for (k=0 ; k<NumOfDigits ; k++) {
//		(*DC)[k]->Print() ;
		HQQ->Fill((*DC)[k]->Q()) ;
		HQT->Fill((*DC)[k]->T()) ;
		}

	}

void E391aCalorimeterFilter::End() {
//	G4cout.form("CalorimeterFilter::End() \n") ;
//	G4cout.flush() ;
	Gin.DeleteAll() ;
	G.DeleteAll() ;
// ...  //////////////////////////////////////		
//	DataFile->Print() ;
//	DataFile->Write() ;
// ...  //////////////////////////////////////		
	}

void E391aCalorimeterFilter::GammaSearch() {
	int kMax=-1 ;
	for (int k=0 ; k<10 ; k++) {
		if (NumOfUsedD>=NumOfDigits) break ;
		kMax=FindMax() ;
		if (kMax==-1) break ;
		G.Add(GammaAroundMax(kMax)) ;
		}
	}

void E391aCalorimeterFilter::ProcessEvent(E391aPrimaryGeneratorAction *Gen) {
//	G4cout.form("E391aCalorimeterFilter::XFun1() \n") ;
	int k ;
	if (NumOfDigits==0) return ; // No Digits at All ...
// G4cout<<1 ; G4cout.flush() ;
	if (HC==0) return ;
	if (NumOfHitsEC==0) return ;
	for (k=0 ; k<NumOfHitsEC ; k++) {
//		(*HC)[k]->Print() ;
		ZHitXY->Fill((*HC)[k]->X()/cm,(*HC)[k]->Y()/cm) ;
		ZHitXP->Fill((*HC)[k]->XPrime()) ;
		ZHitYP->Fill((*HC)[k]->YPrime()) ;
		Gin.Add(new Gamma((*HC)[k]->X(),(*HC)[k]->Y(),(*HC)[k]->KineticEnergy(),(*HC)[k]->XPrime(),(*HC)[k]->YPrime(),(*HC)[k]->Time())) ;
		}
// Initialize Generated Gammas ...
	for (k=0 ; k<Gin.NumOf() ; k++) Gin[k]->InitMomentum(Gen->GenKLongDecayPoint().z()) ;
// Reconstruction All Gammas
	GammaSearch() ;
// ...
	for (k=0 ; k<G.NumOf() ; k++) ECSp->Fill(Gin[k]->E()/GeV) ;

	if (G.NumOf()!=Gin.NumOf()) return ;
	for (k=0 ; k<G.NumOf() ; k++) {
		double X=G[k]->X() , Y=G[k]->Y() ;
		double Xr=Loc(X) ,   Yr=Loc(Y) ;
		double Xc=Cnt(X) ,   Yc=Cnt(Y) ;
		HXoXr->Fill(Loc(Gin[k]->X()),Xr) ;
		HYoYr->Fill(Loc(Gin[k]->Y()),Yr) ;
		HXrS->Fill(X-Gin[k]->X()) ;
		HYrS->Fill(Y-Gin[k]->Y()) ;
		HXY->Fill(Gin[k]->X()/cm,Gin[k]->Y()/cm) ;
		HXYr->Fill(X/cm,Y/cm) ;
		HDx->Fill(G[k]->Dx()) ;
		HDy->Fill(G[k]->Dy()) ;
		HDxDy->Fill(G[k]->Dx(),G[k]->Dy()) ;
		HDxES->Fill(G[k]->Dx(),(G[k]->E()-Gin[k]->E())/GeV) ;
		HDyES->Fill(G[k]->Dy(),(G[k]->E()-Gin[k]->E())/GeV) ;
//		if (!Config->AtEdge(Config->XNo(X),Config->YNo(Y))) {
//			HDxo->Fill(G[k]->Dx()) ;
//			HDyo->Fill(G[k]->Dy()) ;
//			}
		HEo->Fill(Gin[k]->E()/GeV) ;
		HEr->Fill(G[k]->E()/GeV) ;
		HEoEr->Fill(Gin[k]->E()/GeV,G[k]->E()/GeV) ;
		HEoRes->Fill(Gin[k]->E()/GeV,(G[k]->E()-Gin[k]->E())/Gin[k]->E()) ;
		HErS->Fill((G[k]->E()-Gin[k]->E())/GeV) ;
		}
// For reconstructed pio ...
	if (Gin.NumOf()!=2 || G.NumOf()!=2) return ;
//++	if (Distance(G[0],Gin[0])>Distance(G[0],Gin[1])) G.Swap(0,1) ;
	G4double KZ=KLongDecayPointZ(G[0],G[1]) ;
	for (k=0 ; k<G.NumOf() ; k++) G[k]->InitMomentum(KZ) ;
// Minimal Cuts ...
	if (Distance(G[0],G[1])<25*cm) return ;
//	if (KZ<275*cm || KZ>550*cm) return ;
// ...
	HZr->Fill(KZ/cm) ;
	HZrS->Fill(KZ/cm-Gen->GenKLongDecayPoint().z()/cm) ;
	HZoZr->Fill(Gen->GenKLongDecayPoint().z()/cm,KZ/cm) ;

	HAcoRes->Fill(AcoplanarityAngle(Gin[0],Gin[1])/deg,AcoplanarityAngle(G[0],G[1])/deg) ;
	HPTRes->Fill(PionPT(Gin[0],Gin[1]),PionPT(G[0],G[1])) ;
	HIgg->Fill(Distance(Gin[0],Gin[1])/cm) ;
	HIptAco->Fill(PionPT(Gin[0],Gin[1]),AcoplanarityAngle(Gin[0],Gin[1])/deg) ;
	HPgg->Fill(Distance(G[0],G[1])/cm) ;
	HPptAco->Fill(PionPT(G[0],G[1]),AcoplanarityAngle(G[0],G[1])/deg) ;
	G4double	dpt=PionPT(G[0],G[1])-Gen->PionPT() ;
	G4double	dac=AcoplanarityAngle(G[0],G[1])-AcoplanarityAngle(Gin[0],Gin[1]) ;
	HPTS->Fill(dpt) ;
	HAcoS->Fill(dac/deg) ;
	HPrSAc->Fill(dpt,AcoplanarityAngle(G[0],G[1])/deg) ;
	HPrSAcS->Fill(dpt,dac/deg) ;
	HAcSPr->Fill(dac/deg,PionPT(G[0],G[1])) ;
	HPErS->Fill((G[0]->E()+G[1]->E()-Gin[0]->E()-Gin[1]->E())/GeV) ;

	HQQD->Fill((G[0]->E()-G[1]->E())/GeV) ;
	HQTD->Fill(G[0]->T()-G[1]->T()) ;
	HQHD->Fill((G[0]->Theta()-G[1]->Theta())/deg) ;

	}

G4int	E391aCalorimeterFilter::FindMax() {
	G4double	Q=0 ;
	G4int		Index=-1 ;
	for (int k=0 ; k<NumOfDigits ; k++) {
		if (UsedD[k]!=0) continue ;
		G4int id=(*DC)[k]->DetectorID() ;
		if ((*DC)[k]->Q()>Q) {
			Q=(*DC)[k]->Q() ;
			Index=k ;
			}
		}
	return Index ;
	}

Gamma*	E391aCalorimeterFilter::GammaAroundMax(G4int ko) {
	G4double	Xr=0 , Yr=0 , Er=0 , Dx=0 , Dy=0 ;
	int	j=0 ;
	int	kxo=Config->XNo((*DC)[ko]->DetectorID()) ;
	int	kyo=Config->YNo((*DC)[ko]->DetectorID()) ;
	for (int k=0 ; k<NumOfDigits ; k++) {
		if (UsedD[k]!=0) continue ;
		G4int id=(*DC)[k]->DetectorID() ;
		int	kx=Config->XNo(id) ;
		int	ky=Config->YNo(id) ;
		double x=Config->XCenter(kx) ;
		double y=Config->YCenter(ky) ;
		if (abs(kx-kxo)>1) continue ;
		if (abs(ky-kyo)>1) continue ;
//		G4double E=(*DC)[k]->Q()*QDCScale ;
		G4double E=(*DC)[k]->Q()*cc[Config->Index(kx,ky)] ;
		Er+=E ;
		Xr+=E*x ;
		Yr+=E*y ;
		Dx+=E*x*x ;
		Dy+=E*y*y ;
		UsedD[k]=1 ;
		j++ ;
		}
// Test for edge ...
	if (Config->AtEdge(kxo,kyo)) return 0 ;
//+	if (j<=1) return 0 ;
	if (Er<EgCut) return 0 ;
	Xr/=Er ;
	Yr/=Er ;
	Dx=sqrt(Dx/Er-Xr*Xr) ;
	Dy=sqrt(Dy/Er-Yr*Yr) ;
	Xr=Corr(Xr) ;
	Yr=Corr(Yr) ;
	return (new Gamma(Xr,Yr,Er,Dx,Dy,(*DC)[ko]->T())) ;
	}

Gamma*	E391aCalorimeterFilter::GammaAroundMax2(G4int ko) {
	G4double	Xr=0 , Yr=0 , Er=0 , ErSQ=0 ;
	int	j=0 ;
	int	kxo=Config->XNo((*DC)[ko]->DetectorID()) ;
	int	kyo=Config->YNo((*DC)[ko]->DetectorID()) ;
	for (int k=0 ; k<NumOfDigits ; k++) {
		if (UsedD[k]!=0) continue ;
		G4int id=(*DC)[k]->DetectorID() ;
		int	kx=Config->XNo(id) ;
		int	ky=Config->YNo(id) ;
		if (abs(kx-kxo)>1) continue ;
		if (abs(ky-kyo)>1) continue ;
		G4double E=(*DC)[k]->Q()*QDCScale ;
		Er+=E ;
		ErSQ+=sqrt(E) ;
		Xr+=sqrt(E)*Config->XCenter(kx) ;
		Yr+=sqrt(E)*Config->YCenter(ky) ;
		UsedD[k]=1 ;
		j++ ;
		}
	if (j<=1) return 0 ;
	if (Er<EgCut) return 0 ;
	return (new Gamma(Xr/ErSQ,Yr/ErSQ,Er,(*DC)[ko]->T())) ;
	}

G4double E391aCalorimeterFilter::KLongDecayPointZ(Gamma *g1,Gamma *g2) {
	if (g1==0 || g2==0) return (-1) ;
const	static	G4double	Mpio=134.98*MeV ; // Local Constant
	r1=g1->X()*g1->X()+g1->Y()*g1->Y() ;
	r2=g2->X()*g2->X()+g2->Y()*g2->Y() ;
	G4double	o=1.-Mpio*Mpio/g1->E()/g2->E()/2. ;
	G4double	o2=o*o ;
	G4double	a=(o2-1) ;
	G4double	r12=g1->X()*g2->X()+g1->Y()*g2->Y() ;
	G4double	bx=-2*r12+o2*(r1+r2) ;
	G4double	c=-r12*r12+r1*r2*o2 ;
	G4double	b4=bx*bx-4.*a*c ;
	G4double	D2=(-bx-sqrt(b4))/2/a ;
	if (D2<0.0) return (-1) ;
	Dist=sqrt(D2) ;
	return Zc-Dist ;
	}

G4double E391aCalorimeterFilter::AcoplanarityAngle(Gamma *g1,Gamma *g2) {
	if (g1==0 || g2==0) return (0) ;
	G4double	a=atan2(g1->Y(),g1->X())-atan2(g2->Y(),g2->X()) ;
	if (a<180*deg) a=180*deg-a ;
	else a-=180*deg ;
	return a ;
	}

G4double E391aCalorimeterFilter::PionPT(Gamma *g1,Gamma *g2) {
	if (Dist<0.1) KLongDecayPointZ(g1,g2) ;
	G4double	v1R=sqrt(r1+Dist*Dist) ;
	G4double	v2R=sqrt(r2+Dist*Dist) ;
	G4ThreeVector	g1LabR(g1->E()*g1->X()/v1R,g1->E()*g1->Y()/v1R,g1->E()*Dist/v1R) ;
	G4ThreeVector	g2LabR(g2->E()*g2->X()/v2R,g2->E()*g2->Y()/v2R,g2->E()*Dist/v2R) ;
	G4double	pt=(g1LabR+g2LabR).perp() ;
	return pt ;
	}

G4double E391aCalorimeterFilter::Distance(Gamma *g1,Gamma *g2) {
	G4double	dx=g1->X()-g2->X() ;
	G4double	dy=g1->Y()-g2->Y() ;
	return sqrt(dx*dx+dy*dy) ;
	}

G4double E391aCalorimeterFilter::Corr(G4double GlobX) {
static	const	G4double b=15.*mm ;
	if(!CorrON) return GlobX ;
	G4double CC=Config->XCenter(GlobX) ;
	G4double Xb=GlobX-CC ;
	G4double Xc=b*asinh(Xb/Config->DD()*2*sinh(Config->DD()/2/b)) ;
	return Xc+CC ;
	}

// EOF
