// File:	E391aPhysicsList.cc
// Date:	17-Jun-2001 G.Macharashvili Dubna

#include "E391aPhysicsList.hh"

E391aPhysicsList::E391aPhysicsList() {
//	defaultCutValue=0.1*mm ;
	defaultCutValue=100*mm ;
	SetVerboseLevel(0) ;
	if (RootMsg) RootMsg->SpeedInfo("Physics List Started ... ") ;
	}

E391aPhysicsList::~E391aPhysicsList() {
	delete PType ;
	}

void E391aPhysicsList::ConstructParticle() {
//	ConstructAllBosons() ;
 //	ConstructAllLeptons() ;
 //	ConstructAllMesons() ;
 //	ConstructAllBaryons() ;
//	ConstructAllIons() ;
//	ConstructAllShortLiveds() ;
	PType=new E391aPType() ;
	}

void E391aPhysicsList::ConstructProcess() {
//+ All these have to be opened if needed ...
	AddTransportation() ;
//	AddParameterization() ; // Parametrization ... CLOSED
	ConstructEM() ;
	ConstructHad() ;
	ConstructGeneral() ;
//	Print() ;
	}

void E391aPhysicsList::ConstructEM() {
	theParticleIterator->reset() ;
	while( (*theParticleIterator)() ) {
	G4ParticleDefinition*	P=theParticleIterator->value();
	G4ProcessManager*	PM=P->GetProcessManager();
	G4String		PName=P->GetParticleName();

// XXXXXXXXXXXXXXX 
	if (PName == "gamma") {
//		PM->AddProcess(new G4UserSpecialCuts("Gamma-V1"),0,-1,1) ;
//		PM->AddProcess(new G4UserSpecialCuts("Gamma-EC"),0,-1,2) ;
//		PM->AddProcess(new G4UserSpecialCuts("Gamma-BB"),0,-1,3) ;
//		PM->AddProcess(new G4UserSpecialCuts("Gamma-SB"),0,-1,4) ;
		PM->AddDiscreteProcess(new G4GammaConversion()) ;
		PM->AddDiscreteProcess(new G4ComptonScattering()) ;
		PM->AddDiscreteProcess(new G4PhotoElectricEffect()) ;
		} 

// XXXXXXXXXXXXXXX 
	else if (PName=="e-") {
		G4VProcess*	EminusMS=new G4MultipleScattering() ;
		G4VProcess*	EminusIO=new G4eIonisation() ;
		G4VProcess*	EminusBR=new G4eBremsstrahlung() ;
	// add processes
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForElectron"),0,-1,1) ;
		PM->AddProcess(EminusMS) ;
		PM->AddProcess(EminusIO) ;
		PM->AddProcess(EminusBR) ;
	// set ordering for AlongStepDoIt
		PM->SetProcessOrdering(EminusMS,idxAlongStep,1) ;
		PM->SetProcessOrdering(EminusIO,idxAlongStep,2) ;
	// set ordering for PostStepDoIt
		PM->SetProcessOrdering(EminusMS,idxPostStep,1) ;
		PM->SetProcessOrdering(EminusIO,idxPostStep,2) ;
		PM->SetProcessOrdering(EminusBR,idxPostStep,3) ;
		} 

// XXXXXXXXXXXXXXX 
	else if (PName=="e+") {
		G4VProcess* EplusMS=new G4MultipleScattering() ;
		G4VProcess* EplusIO=new G4eIonisation() ;
		G4VProcess* EplusBR=new G4eBremsstrahlung() ;
		G4VProcess* EplusAN=new G4eplusAnnihilation() ;
	// add processes
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForPositron"),0,-1,1) ;	  
		PM->AddProcess(EplusMS) ;
		PM->AddProcess(EplusIO) ;
		PM->AddProcess(EplusBR) ;
		PM->AddProcess(EplusAN) ;
	// set ordering for AtRestDoIt
		PM->SetProcessOrderingToFirst(EplusAN,idxAtRest) ;
	// set ordering for AlongStepDoIt
		PM->SetProcessOrdering(EplusMS,idxAlongStep,1) ;
		PM->SetProcessOrdering(EplusIO,idxAlongStep,2) ;
	// set ordering for PostStepDoIt
		PM->SetProcessOrdering(EplusMS,idxPostStep,1) ;
		PM->SetProcessOrdering(EplusIO,idxPostStep,2) ;
		PM->SetProcessOrdering(EplusBR,idxPostStep,3) ;
		PM->SetProcessOrdering(EplusAN,idxPostStep,4) ;
		} 

// XXXXXXXXXXXXXXX 
	else if( PName=="mu+" || PName=="mu-") {
	// Construct processes for muon+
		G4VProcess* MS=new G4MultipleScattering ;
		G4VProcess* IO=new G4MuIonisation ;
		G4VProcess* BR=new G4MuBremsstrahlung ;
		G4VProcess* PP=new G4MuPairProduction ;
	// add processes
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForMuon"),0,-1,1) ;	  
		PM->AddProcess(MS);
		PM->AddProcess(IO);
		PM->AddProcess(BR);
		PM->AddProcess(PP);
	// set ordering for AlongStepDoIt
		PM->SetProcessOrdering(MS,idxAlongStep,1) ;
		PM->SetProcessOrdering(IO,idxAlongStep,2) ;
	// set ordering for PostStepDoIt
		PM->SetProcessOrdering(MS,idxPostStep,1) ;
		PM->SetProcessOrdering(IO,idxPostStep,2) ;
		PM->SetProcessOrdering(BR,idxPostStep,3) ;
		PM->SetProcessOrdering(PP,idxPostStep,4) ;
		} 

// XXXXXXXXXXXXXX 
	else if ((!P->IsShortLived()) && (P->GetPDGCharge()!=0) && (P->GetParticleName()!="chargedgeantino")) {
	// all others charged Ps except geantino
		G4VProcess* MS=new G4MultipleScattering ;
		G4VProcess* IO=new G4hIonisation ;
	// add processes
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForOthers"),0,-1,1) ;	  
		PM->AddProcess(MS) ;
		PM->AddProcess(IO) ;
	// set ordering for AlongStepDoIt
		PM->SetProcessOrdering(MS,idxAlongStep,1) ;
		PM->SetProcessOrdering(IO,idxAlongStep,2) ;
	// set ordering for PostStepDoIt
		PM->SetProcessOrdering(MS,idxPostStep,1) ;
		PM->SetProcessOrdering(IO,idxPostStep,2) ;
		}
	}
}

void E391aPhysicsList::ConstructHad() {
	G4HadronElasticProcess	*theElasticProcess=new G4HadronElasticProcess ;
	G4LElastic		*theElasticModel=new G4LElastic ;
G4cout<<"Hadrinic Elastic Min Energy:"<<theElasticModel->GetMinEnergy()<<endl ;
	theElasticProcess->RegisterMe(theElasticModel) ;
	theParticleIterator->reset() ;
	while ((*theParticleIterator)()) {
		G4ParticleDefinition*	P=theParticleIterator->value() ;
		G4ProcessManager*	PM=P->GetProcessManager() ;
		G4String		PName=P->GetParticleName() ;

// XXXXXXXXXXXXXX 
	if (PName=="pi+") {
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForPionPlus"),0,-1,1) ;	  
		PM->AddDiscreteProcess(theElasticProcess) ;
		G4PionPlusInelasticProcess	*InelProc=new G4PionPlusInelasticProcess("inelastic") ;
		G4LEPionPlusInelastic		*LEInelModel=new G4LEPionPlusInelastic ;
G4cout<<"pi+ Hadrinic Inelastic Min Energy:"<<LEInelModel->GetMinEnergy()<<endl ;
//				LEInelModel->SetMinEnergy(1*MeV) ;
//		G4HEPionPlusInelastic		*HEInelModel=new G4HEPionPlusInelastic ;
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
		PM->AddDiscreteProcess(InelProc) ;
		}
// XXXXXXXXXXXXXX 
	else if (PName=="pi-") {
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForPionMinus"),0,-1,1) ;	  
		PM->AddDiscreteProcess(theElasticProcess) ;
		G4PionMinusInelasticProcess	*InelProc=new G4PionMinusInelasticProcess("inelastic") ;
		G4LEPionMinusInelastic		*LEInelModel=new G4LEPionMinusInelastic ;
G4cout<<"pi- Hadrinic Inelastic Min Energy:"<<LEInelModel->GetMinEnergy()<<endl ;
//				LEInelModel->SetMinEnergy(1*MeV) ;
//		G4HEPionMinusInelastic		*HEInelModel=new G4HEPionMinusInelastic ;
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
		PM->AddDiscreteProcess(InelProc) ;
		}

// XXXXXXXXXXXXXX 
/*
      else if (PName == "kaon+") {
         PM->AddDiscreteProcess(theElasticProcess);
         G4KaonPlusInelasticProcess* theInelasticProcess = 
                                  new G4KaonPlusInelasticProcess("inelastic");
         G4LEKaonPlusInelastic* theLEInelasticModel = 
                                  new G4LEKaonPlusInelastic;
         theInelasticProcess->RegisterMe(theLEInelasticModel);
         G4HEKaonPlusInelastic* theHEInelasticModel = 
                                  new G4HEKaonPlusInelastic;
         theInelasticProcess->RegisterMe(theHEInelasticModel);
         PM->AddDiscreteProcess(theInelasticProcess);
      }
*/

// XXXXXXXXXXXXXX 
/*
      else if (PName == "kaon-") {
         PM->AddDiscreteProcess(theElasticProcess);
         G4KaonMinusInelasticProcess* theInelasticProcess = 
                                 new G4KaonMinusInelasticProcess("inelastic");
         G4LEKaonMinusInelastic* theLEInelasticModel = 
                                 new G4LEKaonMinusInelastic;
         theInelasticProcess->RegisterMe(theLEInelasticModel);
         G4HEKaonMinusInelastic* theHEInelasticModel = 
                                 new G4HEKaonMinusInelastic;
         theInelasticProcess->RegisterMe(theHEInelasticModel);
         PM->AddDiscreteProcess(theInelasticProcess);
      }
*/

// XXXXXXXXXXXXXX 
/*
       else if (PName == "kaon0S") {
          PM->AddDiscreteProcess(theElasticProcess);
          G4KaonZeroSInelasticProcess* theInelasticProcess = 
                              new G4KaonZeroSInelasticProcess("inelastic");
          G4LEKaonZeroSInelastic* theLEInelasticModel = 
                              new G4LEKaonZeroSInelastic;
          theInelasticProcess->RegisterMe(theLEInelasticModel);
          G4HEKaonZeroInelastic* theHEInelasticModel = 
                              new G4HEKaonZeroInelastic;
          theInelasticProcess->RegisterMe(theHEInelasticModel);
          PM->AddDiscreteProcess(theInelasticProcess);
       }
*/

// -------------------- KaonZeroLong 26-Jan-2001 G.Macharashvili Juelich
// For All hadrons !!!
// E < 20 MeV case inelastic LE (Low Energy Model)
// E > 20 GeV case inelastic HE (High Energy Model)

// XXXXXXXXXXXXXX 
	else if (PName=="kaon0L") {
		G4KaonZeroLInelasticProcess	*InelProc=new G4KaonZeroLInelasticProcess("inelastic") ;
		G4LEKaonZeroLInelastic		*LEInelModel=new G4LEKaonZeroLInelastic ;
G4cout<<"KL  Hadrinic Inelastic Min Energy:"<<LEInelModel->GetMinEnergy()<<endl ;
//				LEInelModel->SetMinEnergy(5*MeV) ;
//		G4HEKaonZeroInelastic		*HEInelModel=new G4HEKaonZeroInelastic ;
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForKaonZero"),0,-1,1) ;	  
		PM->AddDiscreteProcess(theElasticProcess) ;
		PM->AddDiscreteProcess(InelProc) ;
		}

// XXXXXXXXXXXXXX 
	else if (PName=="proton") {
		G4ProtonInelasticProcess	*InelProc=new G4ProtonInelasticProcess("inelastic") ;
		G4LEProtonInelastic		*LEInelModel=new G4LEProtonInelastic ;
G4cout<<"p   Hadrinic Inelastic Min Energy:"<<LEInelModel->GetMinEnergy()<<endl ;
//				LEInelModel->SetMinEnergy(5*MeV) ;
//		G4HEProtonInelastic		*HEInelModel=new G4HEProtonInelastic ; 
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForProton"),0,-1,1) ;	  
		PM->AddDiscreteProcess(theElasticProcess) ; 
		PM->AddDiscreteProcess(InelProc) ;
		}

// XXXXXXXXXXXXXX 
	else if (PName=="anti_proton") {
		G4AntiProtonInelasticProcess* InelProc=new G4AntiProtonInelasticProcess("inelastic") ;
		G4LEAntiProtonInelastic* LEInelModel=new G4LEAntiProtonInelastic ;
//		G4HEAntiProtonInelastic* HEInelModel=new G4HEAntiProtonInelastic ;
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
		PM->AddDiscreteProcess(theElasticProcess) ;
		PM->AddDiscreteProcess(InelProc) ;
		}

// XXXXXXXXXXXXXX 
	else if (PName=="neutron") {
		G4NeutronInelasticProcess	*InelProc=new G4NeutronInelasticProcess("inelastic") ;
		G4LENeutronInelastic		*LEInelModel=new G4LENeutronInelastic ;
G4cout<<"n   Hadrinic Inelastic Min Energy:"<<LEInelModel->GetMinEnergy()<<endl ;
//				LEInelModel->SetMinEnergy(1*MeV) ;
//		G4HENeutronInelastic		*HEInelModel=new G4HENeutronInelastic ;
		InelProc->RegisterMe(LEInelModel) ;
//		InelProc->RegisterMe(HEInelModel) ;
// 		PM->AddProcess(new G4UserSpecialCuts("CutsForNeutron"),0,-1,1) ;	  
		PM->AddDiscreteProcess(theElasticProcess) ;
		PM->AddDiscreteProcess(InelProc) ;
		}

// XXXXXXXXXXXXXX 
/*
       else if (PName == "anti_neutron") {
          PM->AddDiscreteProcess(theElasticProcess);
          G4AntiNeutronInelasticProcess* theInelasticProcess = 
                                new G4AntiNeutronInelasticProcess("inelastic");
          G4LEAntiNeutronInelastic* theLEInelasticModel = 
                                new G4LEAntiNeutronInelastic;
          theInelasticProcess->RegisterMe(theLEInelasticModel);
          G4HEAntiNeutronInelastic* theHEInelasticModel = 
                                new G4HEAntiNeutronInelastic;
          theInelasticProcess->RegisterMe(theHEInelasticModel);
          PM->AddDiscreteProcess(theInelasticProcess);
       }
*/

// XXXXXXXXXXXXXX 
/*
       else if (PName == "lambda") {
          PM->AddDiscreteProcess(theElasticProcess);
          G4LambdaInelasticProcess* theInelasticProcess = 
                                     new G4LambdaInelasticProcess("inelastic");
          G4LELambdaInelastic* theLEInelasticModel = new G4LELambdaInelastic;
          theInelasticProcess->RegisterMe(theLEInelasticModel);
          G4HELambdaInelastic* theHEInelasticModel = new G4HELambdaInelastic;
          theInelasticProcess->RegisterMe(theHEInelasticModel);
          PM->AddDiscreteProcess(theInelasticProcess);
       }
*/

// XXXXXXXXXXXXXX 
/*
       else if (PName=="anti_lambda") {
          PM->AddDiscreteProcess(theElasticProcess);
          G4AntiLambdaInelasticProcess* theInelasticProcess = 
                                 new G4AntiLambdaInelasticProcess("inelastic");
          G4LEAntiLambdaInelastic* theLEInelasticModel = 
                                 new G4LEAntiLambdaInelastic;
          theInelasticProcess->RegisterMe(theLEInelasticModel);
          G4HEAntiLambdaInelastic* theHEInelasticModel = 
                                 new G4HEAntiLambdaInelastic;
          theInelasticProcess->RegisterMe(theHEInelasticModel);
          PM->AddDiscreteProcess(theInelasticProcess);
       }
*/
    }
}

#include "G4Decay.hh"
void E391aPhysicsList::ConstructGeneral() {
// Add Decay Process
	G4Decay	*DecProc=new G4Decay() ;
	theParticleIterator->reset() ;
	while( (*theParticleIterator)() ) {
		G4ParticleDefinition *P=theParticleIterator->value();
		G4ProcessManager *PM=P->GetProcessManager();
		if (DecProc->IsApplicable(*P)) { 
			PM->AddProcess(DecProc);
// set ordering for PostStepDoIt and AtRestDoIt
			PM->SetProcessOrdering(DecProc,idxPostStep);
			PM->SetProcessOrdering(DecProc,idxAtRest);
			}
		}
	}

void E391aPhysicsList::Print() {
	theParticleIterator->reset() ;
	while( (*theParticleIterator)() ) {
		G4ParticleDefinition *P=theParticleIterator->value();
		P->GetProcessManager()->DumpInfo() ;
		}
	}

void E391aPhysicsList::AddTranspotation() {
	G4Transportation *TransProc=new G4Transportation() ;
	theParticleIterator->reset() ;
	while ((*theParticleIterator)()) {
		G4ParticleDefinition *P=theParticleIterator->value() ;
		G4ProcessManager *PM=P->GetProcessManager() ;
		if(!P->IsShortLived()) {
			PM->AddProcess(TransProc) ;
			PM->SetProcessOrderingToFirst(TransProc,idxAlongStep) ;
			}
		}
	}

void E391aPhysicsList::AddParameterization() {
	G4FastSimulationManagerProcess *FSProc=new G4FastSimulationManagerProcess() ;
	theParticleIterator->reset() ;
	while ((*theParticleIterator)()) {
		G4ParticleDefinition *P=theParticleIterator->value() ;
		G4ProcessManager *PM=P->GetProcessManager() ;
		PM->AddProcess(FSProc,-1,1,1) ;
		PM->SetProcessOrderingToFirst(FSProc,idxAlongStep) ;
		}
	}

void E391aPhysicsList::SetCuts() {
// set cut values for gamma at first and for e- second and next for e+,
// because some processes for e+/e- need cut values for gamma 
	SetCutValue(1.0,"gamma");
	SetCutValue(1.0,"e-");
	SetCutValue(1.0,"e+");
	SetCutValue(1*cm,"mu-");
	SetCutValue(1*cm,"mu+");
// set cut values for proton and anti_proton before all other hadrons
// because some processes for hadrons need cut values for proton/anti_proton 
	SetCutValue(1.0,"proton");
	SetCutValue(1.0,"anti_proton");
	SetCutValue(1.0,"neutron");
	SetCutValue(1*cm,"pi-");
	SetCutValue(1*cm,"pi+");
  
//	SetCutValueForOthers(defaultCutValue);

//	DumpCutValues("gamma");
//	DumpCutValues("e+");
//	DumpCutValues("e-");
//	DumpCutValues("neutron");

// Go back to default values of cuts ...
	SetCutsWithDefault() ;
//	DumpCutValuesTable() ;
// Print ... ON/Off
	SetVerboseLevel(0) ;
	}

/*
void E391aPhysicsList::ConstructAllBosons() {
	G4BosonConstructor C ;
	C.ConstructParticle() ;
	}
*/
void E391aPhysicsList::ConstructAllLeptons() {
	G4LeptonConstructor C ;
	C.ConstructParticle() ;
	}

void E391aPhysicsList::ConstructAllMesons() {
	G4MesonConstructor C ;
	C.ConstructParticle() ;
	}

void E391aPhysicsList::ConstructAllBaryons() {
	G4BaryonConstructor C ;
	C.ConstructParticle() ;
	}
/*
void E391aPhysicsList::ConstructAllIons() {
	G4IonConstructor C ;
	C.ConstructParticle() ;
	}

void E391aPhysicsList::ConstructAllShortLiveds() {
	G4ShortLivedConstructor C ;
	C.ConstructParticle() ;
	}
*/

// EOF
