// File:	Generator2.cc
// date:	21-May-2001 G.Macharashvili Dubna

#include "E391aPrimaryGeneratorAction.hh"

#include <time.h>

E391aPrimaryGeneratorAction::E391aPrimaryGeneratorAction() {

	PGun=new G4ParticleGun(1) ;
	Beam=new E391aBeam ;
	PosGen=new E391aDecayPos(Zmin,Zmax) ;
	PreTrigger=new E391aPrimaryTrigger() ;

	G4long	t=time(0) ;
//	t=0 ; ///////////////////////////////////////////////// remove
	HepRandom::setTheSeed(t) ;

	G4ParticleTable* PartTab=G4ParticleTable::GetParticleTable() ;
	KLong=PartTab->FindParticle("kaon0L") ;
	Pion0=PartTab->FindParticle("pi0") ;
	G    =PartTab->FindParticle("gamma") ;

// Create new KLong decay channels ...
	DT=KLong->GetDecayTable() ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",1.,3,"pi0","nu_e","anti_nu_e")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",0.000927,2,"pi0","pi0")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",0.000586,2,"gamma","gamma")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",1.68e-6,3,"pi0","gamma","gamma")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",2.06e-3,2,"pi+","pi-")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",4.61e-5,3,"pi+","pi-","gamma")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",2.4e-7,3,"gamma","gamma","gamma")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",5.6e-6,3,"pi0","pi0","gamma")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",3.62e-3/2,4,"pi+","e-","gamma","anti_nu_e")) ;
	DT->Insert(new G4PhaseSpaceDecayChannel("kaon0L",3.62e-3/2,4,"pi-","e+","gamma","nu_e")) ;
// Forced Decay Channels ...
	KC=KLong->GetDecayTable()->GetDecayChannel(0) ;
	PC=Pion0->GetDecayTable()->GetDecayChannel(0) ;
// Print decay table ...
	KLong->GetDecayTable()->DumpInfo() ;
//	Pion0->GetDecayTable()->DumpInfo() ;

// Hist Initialization ...
/*
	HBeamProfXY=	new TH2F("BeamProf-XY","Beam Profile XY",40,-4,+4,40,-4,+4) ;
	HBeamProfXZ=	new TH2F("BeamProf-XZ","Beam Profile XZ",200,250,650,40,-4,+4) ;
	HKLongDecayZ=	new TH1F("KLongDecayZ","KLong Decay PointZ",200,250,650) ;

	HPionPT=	new TH1F("PionPT","Pion PT",100,0,250) ;
	HPionPhi=	new TH1F("PionPhi","Pion Phi",90,-180,180) ;
	HPionCosTh=	new TH1F("PionCosTh","Pion Cos Theta",100,-1,1) ;
	HPionPLab=	new TH1F("PionPLab","Pion P Lab",100,0,500) ;

	HGammaPT=	new TH1F("GammaPT","Gamma PT",100,0,250) ;
	HGammaThLab=	new TH1F("GammaThLab","Gamma Theta Lab",100,0,100) ;
*/
	HGammaE=	new TH1F("GammaE","Gamma E Lab",200,0,400) ;
		HGammaE->SetXTitle("#gamma energy,  MeV") ;
		HGammaE->SetYTitle("Number of events") ;
	}

E391aPrimaryGeneratorAction::~E391aPrimaryGeneratorAction() {
	delete PGun ;
	delete Beam ;
	delete PosGen ;
	delete PreTrigger ;
	}

/*
// Single Gamma generation ...
void	E391aPrimaryGeneratorAction::GeneratePrimaries(G4Event* Evt) {
	G4double GEnergy=1*MeV+G4UniformRand()*1000*MeV ;
	G4ThreeVector Vtx=Beam->KLongPos(G4UniformRand()*(Zmax-Zmin)+Zmin) ;
	G4double th=G4UniformRand()*180*deg ;
	G4double fi=G4UniformRand()*360*deg ;
// For CCo3 Simulation ...
const	G4double	XHalf=165*mm ;
	G4double	x=G4UniformRand()*XHalf*2-XHalf ;
	G4double	y=G4UniformRand()*XHalf*2-XHalf ;
	th=atan(sqrt(x*x+y*y)/(5990*mm-Vtx.z())) ;
	fi=atan2(y,x) ;
	G4ThreeVector GOrigin(x,y,5990*mm-0.1*mm) ;
	G4ThreeVector GDirection(sin(th)*cos(fi),sin(th)*sin(fi),cos(th)) ;
	PGun->SetParticleDefinition(G) ;
	PGun->SetParticleEnergy(GEnergy) ;
	PGun->SetParticleMomentumDirection(GDirection) ;
	PGun->SetParticlePosition(GOrigin) ;
	PGun->SetParticleTime(0) ;
	PGun->GeneratePrimaryVertex(Evt) ;
	}
*/

void	E391aPrimaryGeneratorAction::GeneratePrimaries(G4Event* Event) {

	KLongKineticEnergy=Beam->KLongEkin() ;
	G4double E=KLongKineticEnergy+KLong->GetPDGMass() ;
//	KLongDecayPoint=Beam->KLongPos(G4UniformRand()*(Zmax-Zmin)+Zmin) ;
	KLongDecayPoint=Beam->KLongPos(PosGen->KLongZ(sqrt(E*E-KLong->GetPDGMass()*KLong->GetPDGMass())/E)) ;
	KLongDirection=Beam->KLongDir(KLongDecayPoint) ;
	G4DynamicParticle *KD=new G4DynamicParticle(KLong,KLongDirection,KLongKineticEnergy) ;
	K4=KD->Get4Momentum() ;

//	BeamHistFill() ;

// Kaon0L decay ...
do {
	G4DecayProducts* P=KC->DecayIt(KLong->GetPDGMass()) ;
	for (int k=0 ; k<P->entries() ; k++) {
		G4DynamicParticle* p=P->operator[](k) ;
// Remove neutrinos ...
		if (p->GetDefinition()->GetParticleName()=="nu_e") continue ;
		if (p->GetDefinition()->GetParticleName()=="anti_nu_e") continue ;
		if (p->GetDefinition()->GetParticleName()=="nu_mu") continue ;
		if (p->GetDefinition()->GetParticleName()=="anti_nu_mu") continue ;
		if (p->GetDefinition()->GetParticleName()=="nu_tau") continue ;
		if (p->GetDefinition()->GetParticleName()=="anti_nu_tau") continue ;
// Particle Analysis ...
		G4LorentzVector V4=p->Get4Momentum() ;
		CosThCM=V4.cosTheta() ;
		V4.boost(K4.boostVector()) ;
		p->Set4Momentum(V4) ;
// Pion0 ...
		if (p->GetDefinition()->GetParticleName()=="pi0") {
			X=p->Get4Momentum() ;
//			HPionHistFill() ;
// Pion0 decays ...
// Pion0 decay according BR:
			G4DecayProducts* W=Pion0->GetDecayTable()->SelectADecayChannel()->DecayIt(Pion0->GetPDGMass()) ;
// Pion0 decay via selected channel (0 or 1) PC see above !...:
// ++			G4DecayProducts* W=PC->DecayIt(Pion0->GetPDGMass()) ;
// Pion0 Decay end ...
			for (int j=0 ; j<W->entries() ; j++) {
				G4DynamicParticle* g=W->operator[](j) ;
				G4LorentzVector V4g=g->Get4Momentum() ;
				V4g.boost(X.boostVector()) ;
				g->Set4Momentum(V4g) ;
HGammaE->Fill(g->GetKineticEnergy()) ;
				NP.Add(g) ;
				}
			delete W ;
			}
		else {
			NP.Add(p) ;
			}
		}
	delete P ;
	} while (PreTrigger->Response(KLongDecayPoint,NP)!=true) ;
	for (int k=0 ; k<NP.NumOf() ; k++) RegisterParticle(Event,KLongDecayPoint,NP[k]) ;
	NP.Clear() ;

	delete KD ;
	return ;
	}


void	E391aPrimaryGeneratorAction::RegisterParticle(G4Event* Evt,G4ThreeVector &Vtx,G4DynamicParticle *P) {
	PGun->SetParticleDefinition(P->GetDefinition()) ;
	PGun->SetParticleEnergy(P->GetKineticEnergy()) ;
	PGun->SetParticleMomentumDirection(P->GetMomentumDirection()) ;
	PGun->SetParticlePosition(Vtx) ;
	PGun->SetParticleTime(0) ;
	PGun->GeneratePrimaryVertex(Evt) ;
// Save for record ...
	}

void	E391aPrimaryGeneratorAction::PionHistFill() {
	HPionPT->Fill(X.perp()) ;
	HPionPhi->Fill(X.phi()/degree) ;
	HPionCosTh->Fill(CosThCM) ;
	HPionPLab->Fill(X.vect().mag()) ;
	}

void	E391aPrimaryGeneratorAction::BeamHistFill() {
	HBeamProfXY->Fill(KLongDecayPoint.x()/cm,KLongDecayPoint.y()/cm) ;
	HBeamProfXZ->Fill(KLongDecayPoint.z()/cm,KLongDecayPoint.x()/cm) ;
	HKLongDecayZ->Fill(KLongDecayPoint.z()/cm) ;
	}

// EOF
