// File:	E391aVSensitiveDetector.cc
// Date:	22-Nov-2001 G.Macharashvili Dubna

#include "E391aVSensitiveDetector.hh"

E391aVSensitiveDetector::E391aVSensitiveDetector(G4String DetName,G4int xNumOfModules)
:
G4VSensitiveDetector(DetName) {
	NumOfModules=xNumOfModules ;
	ModuleSolid=0 ;

	Aft=new G4int [NumOfModules] ;
	for (G4int k=0 ; k<NumOfModules ; k++) Aft[k]=0 ;
	AffineTrans = new G4AffineTransform [NumOfModules] ;
	AffineTransInverse = new G4AffineTransform [NumOfModules] ;
	}

E391aVSensitiveDetector::~E391aVSensitiveDetector() {
	delete [] Aft ;
	delete [] AffineTrans ;
	delete [] AffineTransInverse ;
	}

G4int	E391aVSensitiveDetector::StepClassifier(G4Step* S) {
	TH=(G4TouchableHistory*)(S->GetPreStepPoint()->GetTouchable()) ;
// Event seq num ...
	xEventID=G4RunManager::GetRunManager()->GetCurrentEvent()->GetEventID() ;
// Track seq num ...
	G4Track *T=S->GetTrack() ;
	xTrackID=T->GetTrackID() ;
// Particle type ...
	xParID=E391aPType::PType(T->GetDefinition()) ;
// Define Detector.Name Volume ...
	G4int Depth=-1 , MaxDepth=TH->GetHistory()->GetDepth() ;
	G4VPhysicalVolume* V1 ;
	for (G4int k=0 ; k<MaxDepth ; k++) {
		V1=TH->GetVolume(k) ;
		if (V1->GetName()==GetName()) {
			Depth=k ;
			break ;
			}
		}
	if (Depth==-1) G4cout<<"E391aVSensitiveDetector: Cant find <"<<GetName()<<"> volume ..."<<endl ;
	ModuleNo=V1->GetCopyNo() ;
// Define Transformation ...
	if (Aft[ModuleNo]==0) {
// Current volume. In general isnt the same as detector volume ...
//		G4VPhysicalVolume* V=TH->GetVolume() ;
		if (ModuleSolid==0) ModuleSolid=TH->GetSolid(Depth) ;
		AffineTrans[ModuleNo]=TH->GetHistory()->GetTransform(Depth) ; // Glob -> Local
		if (AffineTrans[ModuleNo].IsRotated()
			|| AffineTrans[ModuleNo].IsTranslated()) Aft[ModuleNo]=1 ;
		AffineTransInverse[ModuleNo]=AffineTrans[ModuleNo].Inverse() ; // Local -> Glob
// .......................................................usage
// PointLocal=AffineTrans[ModuleNo].TransformPoint(PointGlobal)
// VectLocal=AffineTrans[ModuleNo].TransformAxis(VectGlobal)
// ............................................................
		}

	G4ThreeVector P=S->GetPreStepPoint()->GetPosition() ;
	G4ThreeVector P2=S->GetPostStepPoint()->GetPosition() ;
	G4ThreeVector PLoc=AffineTrans[ModuleNo].TransformPoint(P) ;
	G4ThreeVector DP=AffineTrans[ModuleNo].TransformAxis(P2-P)*0.0001*mm ;
	G4ThreeVector PLoc2=PLoc+DP ;
	EInside EI =V1->GetLogicalVolume()->GetSolid()->Inside(PLoc) ;
	EInside EI2=V1->GetLogicalVolume()->GetSolid()->Inside(PLoc2) ;

	switch(EI+EI2*3) {
		default:
		case 0:
		case 3:
		case 6:
			SType=0 ; // Undefined or Error ...
			break ;
		case 4:
			SType=0 ;
			if (S->GetTotalEnergyDeposit()==0) break ;
		case 7:
			SType=1 ; // Volume Entry Step ...
			break ;
		case 1:
		case 2:
		case 5:
		case 8:
			SType=2 ; // Inside Step ...
			break ;
		}
	return SType ;
	}

// EOF
