// File:	EnergyDigitizer.cc
// Date:	27-Nov-2001 G.Macharashvili Juelich

#include "EnergyDigitizer.hh"

EnergyDigitizer::~EnergyDigitizer() {}

EnergyDigitizer::EnergyDigitizer(G4double xScale,G4double xNoise,G4double m2n) {
	Noise=xNoise ;
	Scale=xScale ;
	MeV2Npe=0 ;
	Ea=0.0254 ;
	Eb=0.0125 ;
	if (m2n>0) SetMeV2Npe(m2n) ;
	}

void EnergyDigitizer::SetMeV2Npe(G4double m2n) {
	MeV2Npe=m2n ;
	if (m2n>200) MeV2Npe=200 ;
	else if (m2n<1) MeV2Npe=1 ;
	Ea=Eb=0 ;
	}

void EnergyDigitizer::SetAB(G4double a,G4double b) {
	Ea=a ;
	if (a>0.10) Ea=0.10 ;
	else if (a<0.01) Ea=0.01 ;
	Eb=b ;
	if (b>0.10) Eb=0.10 ;
	else if (b<0.00) Eb=0.00 ;
	MeV2Npe=0 ;
	}

G4double EnergyDigitizer::EnergyRandomize(G4double e) {
	G4double ex=e ;
	if (MeV2Npe>0) {
		ex=RandPoissonQ::shoot(e*MeV2Npe)/MeV2Npe ;
		}
	else if (Ea>0 || Eb>0) {
		ex=RandGaussQ::shoot(e/GeV,sqrt(e/GeV)*Ea+Eb*(e/GeV))*GeV ;
		}
	return (ESigmaON) ? ex : e ;
	}

G4double EnergyDigitizer::PedestalRandomize() { 
	return (PSigmaON) ? rint(RandGaussQ::shoot(0,Noise)) : 0 ;
	}

G4double EnergyDigitizer::operator()(G4double e) {
	e=EnergyRandomize(e) ;
	double	qdc=(ESigmaON) ? rint(e/Scale+PedestalRandomize()) : e/Scale ;
	if (qdc<MinCount) return MinCount ; // protection
	if (qdc>MaxCount) return MaxCount ; // overflow
	else return qdc ;
	}

// EOF
